/*
 * I2S_hello_world.c
 *
 * Created: 11/22/2015 9:17:37 PM
 * Author : Ben Garber
 *
 * SPH0645LM4H-B microphone hello world
 * Based on Neil Gershenfeld's (11/17/15) SPU0414HR5H microphone hello-world code
 *
 * I2S interface code using SPI Copyright Ben Garber 11/2015
 *
 * This work may be reproduced, modified, distributed,
 * performed, and displayed for any purpose. Copyright is
 * retained and must be preserved. The work is provided
 * as is; no warranty is provided, and users accept all
 * liability.
 */
#include <avr/io.h>
#include <util/delay.h>
#include "bitbang_uart_Fab.h"

// For UART
#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_out (1 << 1)
#define NPTS 100 // points in buffer

// For I2S
#define WS_pin (1 << 7)
#define I2S_direction DDRA
#define I2S_port PORTA
#define UCLK_pin (1 << 4)
#define USI_counter_mask 0b1111
#define WS_ready_mask 0b1110
#define WAITTIME 200 // I2S words to wait (hope this is 50ms! Don't care if it isn't!)
#define USI_wait() while(!bit_test(USISR, USIOIF));\
	set(USISR, (1 << USIOIF)) // Waits for the USI to complete reading a byte, clears the flag
#define USI_wait3times() for(j=0;j<3;j++){USI_wait();}

uint8_t j;

inline void USI_get3bytes(uint8_t* incoming_data) {
	for(j = 0; j < 3; j++){
		USI_wait();
		incoming_data[j] = USIBR; // Load USI data buffer into array
	}
}

inline void I2S_toggle_WS( void ){
	while ((USISR & USI_counter_mask) < WS_ready_mask); // Wait until USI counter shows that it's time for WS to toggle
	toggle(I2S_port, WS_pin);
	USI_wait();
}


int main(void) {

	static unsigned char i, audio_low_bytes[NPTS], audio_high_bytes[NPTS]; // (c) Gershenfeld
	static unsigned char temp_USI_data[4]; // (for storing data directly out of USI)
	
	// set clock divider to /1 (c) Gershenfeld
	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	
	// initialize UART pins (c) Gershenfeld
	set(serial_port, serial_pin_out);
	output(serial_direction, serial_pin_out);
	
	// Initialize I2S pins
	output(I2S_direction, WS_pin | UCLK_pin);
	
	// Set up Timer/Counter0 in CTC mode generating 2 MHz, no output
	set(TCCR0A, 0b10 << WGM00);
	set(TCCR0B, 0b001 << CS00);
	
	// Set up USI in three-wire mode
	set(USICR, (1 << USIWM0));
	
	// main loop
	while (1) {
		// send framing
		put_char(&serial_port, serial_pin_out, 1);
		char_delay();
		put_char(&serial_port, serial_pin_out, 2);
		char_delay();
		put_char(&serial_port, serial_pin_out, 3);
		char_delay();
		put_char(&serial_port, serial_pin_out, 4);
		char_delay();
		
		// Set up I2S (wake up mic)
		set(USICR, 1 << USICS0); // Specifies Timer/Counter0 as USI clock source
		
		// Waste a bunch of words
        set(I2S_port, WS_pin);
        for(i = 0; i < WAITTIME; i++){
		    USI_wait3times();
		    I2S_toggle_WS();
		    USI_wait3times();
		    I2S_toggle_WS();
        }
		
		for (i = 0; i < NPTS; i++) { // Collect data
			// Get 3 data bytes
			USI_get3bytes(temp_USI_data);
			// Waste more time
            I2S_toggle_WS();
			// Store said bytes
			audio_low_bytes[i] = temp_USI_data[1];
			audio_high_bytes[i] = temp_USI_data[0];
			
			// Wait for the USI data register to overflow 4 times
			USI_wait3times();
            I2S_toggle_WS();
		}
		// Put mic to sleep
		clear(USICR, 1 << USICS0);
		
		// Send data
		for (i = 0; i < NPTS; ++i) {
			put_char(&serial_port, serial_pin_out, audio_low_bytes[i]);
			put_char(&serial_port, serial_pin_out, audio_high_bytes[i]);
		}
	}
}
